from typing import Optional, List

from pydantic import Field, ConfigDict

from thestage.services.clients.thestage_api.dtos.base_response import TheStageBaseResponse
from thestage.logging.dto.log_message import LogMessage


class UserLogsQueryResponse(TheStageBaseResponse):
    model_config = ConfigDict(use_enum_values=True)

    queryResult: Optional[List[LogMessage]] = Field(None, alias='queryResult')
    entriesFound: Optional[int] = Field(None, alias='entriesFound')
    totalCount: Optional[int] = Field(None, alias='totalCount')
